@@ -1,8 +1,7 @@
-var _arity = /*#__PURE__*/require('./internal/_arity');
+var _arity = require('./internal/_arity');
+var _concat = require('./internal/_concat');
+var _curry2 = require('./internal/_curry2');
 
-var _concat = /*#__PURE__*/require('./internal/_concat');
-
-var _curry2 = /*#__PURE__*/require('./internal/_curry2');
 
 /**
  * `tryCatch` takes two functions, a `tryer` and a `catcher`. The returned
@@ -22,14 +21,11 @@ var _curry2 = /*#__PURE__*/require('./internal/_curry2');
  * @return {Function} A new function that will catch exceptions and send then to the catcher.
  * @example
  *
- *      R.tryCatch(R.prop('x'), R.F)({x: true}); //=> true
- *      R.tryCatch(() => { throw 'foo'}, R.always('catched'))('bar') // => 'catched'
- *      R.tryCatch(R.times(R.identity), R.always([]))('s') // => []
- `` */
-
-
-var tryCatch = /*#__PURE__*/_curry2(function _tryCatch(tryer, catcher) {
-  return _arity(tryer.length, function () {
+ *      R.tryCatch(R.prop('x'), R.F, {x: true}); //=> true
+ *      R.tryCatch(R.prop('x'), R.F, null);      //=> false
+ */
+module.exports = _curry2(function _tryCatch(tryer, catcher) {
+  return _arity(tryer.length, function() {
     try {
       return tryer.apply(this, arguments);
     } catch (e) {
@@ -37,4 +33,3 @@ var tryCatch = /*#__PURE__*/_curry2(function _tryCatch(tryer, catcher) {
     }
   });
 });
-module.exports = tryCatch;
\ No newline at end of file
